#include <linux/soundcard.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#include "ivad.h"
#include "audio_ctl.h"
#include "ivad_config.h"
#include "ivad_ioctl.h"
#include "stdio.h"

static int mixer_fd = -1;

//////////////////////////////////////////////////////////////////
// volume control
//////////////////////////////////////////////////////////////////

// opens the mixer device and returns its file descriptor
int open_mixer_device()
{
	if ((mixer_fd = open(ivad_config.mixer_devicename, O_RDWR)) == -1)
	{
		fprintf(stderr, "Error opening mixer device %s: %s\n", ivad_config.mixer_devicename, strerror(errno));
	}
	return mixer_fd;
}

void close_mixer_device()
{
	if (mixer_fd >= 0)
		close(mixer_fd);
	mixer_fd = -1;
}

// prerequisits: mixer device must be open
// returns -1 on error
int get_volume()
{
	int volume;
	if (gen_ioctl(mixer_fd, MIXER_READ(MIXER_CHANNEL), &volume) < 0)
		return -1;
	return volume;
}

// returns -1 on error
int set_volume(int value)
{
	if (mixer_fd < 0 && open_mixer_device() < 0)
		return -1;
	return set_volume_direct(value);
}

// prerequisits: mixer device must be open
// returns -1 on error
// value is elem of [0;MAX_VOLUME]
int set_volume_direct(int value)
{
	if (gen_ioctl(mixer_fd, MIXER_WRITE(MIXER_CHANNEL), &value) < 0)
		return -1;
	return 0;
}

void set_volume_relative(int value)
{
	int vol, new_vol;
	
	if (mixer_fd < 0 && open_mixer_device() < 0)
		return;

	vol = get_volume();
	new_vol = (vol + value < 0) ? 0 : (vol + value > MAX_VOLUME) ? MAX_VOLUME : vol + value;
	
	set_volume_direct(new_vol);

	dprint1("volume: %d\n", new_vol);
}

void mute(bool action)
{
	static int saved_volume = 0;

	if (mixer_fd < 0 && open_mixer_device() < 0)
		return;

	if (action == true)
	{
		saved_volume = get_volume();
		set_volume_direct(0);
	}
	else if (saved_volume != 0)
		set_volume_direct(saved_volume);
}

void mute_toggle()
{
	if (mixer_fd < 0 && open_mixer_device() < 0)
		return;

	if (!get_volume())
		mute(false);
	else
		mute(true);
}

