#ifndef EVENT_HANDLER_H
#define EVENT_HANDLER_H


///// --- event modes --- /////
#define EVENT_MODE_NONE		0x1
#define EVENT_MODE_BRIGHTNESS	0x2
#define EVENT_MODE_VOLUME	0x4
#define EVENT_MODE_PRIVATE	0x80
#define EVENT_MODE_ALL_WITH_PRIVATE	0xff
#define EVENT_MODE_ALL		(EVENT_MODE_ALL_WITH_PRIVATE & ~EVENT_MODE_PRIVATE)



///// --- event types --- /////

// NONE: used for terminating the event_mapping array
#define EVENT_TYPE_NONE			0

// EXIT: exit the program
#define EVENT_TYPE_EXIT			1

// CHANGE_MODE: change to any mode defined by EVENT_MODE_* except EVENT_MODE_ALL
// 	param: EVENT_MODE_*
#define EVENT_TYPE_CHANGE_MODE		2

// COMMAND: execute a command
// 	param: command
#define EVENT_TYPE_COMMAND		3

// BRIGHTNESS: adjust the lcd brightness
//	param: EVENT_FLAG_INCREASE, EVENT_FLAG_DECREASE
#define EVENT_TYPE_BRIGHTNESS		4

// BRIGHTNESS_SET: set brightness to a fixed value
// 	param: value 0..255
#define EVENT_TYPE_BRIGHTNESS_SET	5

// VOLUME: adjust main volume
//	param: EVENT_FLAG_INCREASE, EVENT_FLAG_DECREASE
#define EVENT_TYPE_VOLUME		6

// VOLUME_SET: set volume to a fixed value
// 	param: value 0..27500
#define EVENT_TYPE_VOLUME_SET		7

// MUTE: mute/unmute. this is not equivalent to VOLUME_SET(0)
//  because MUTE restores the old values
// 	param: EVENT_FLAG_ON, EVENT_FLAG_TOGGLE, EVENT_FLAG_OFF
#define EVENT_TYPE_MUTE			8

// BLUETOTH: change the state of the internal bluetooth device
// 	param: EVENT_FLAG_ON, EVENT_FLAG_TOGGLE, EVENT_FLAG_OFF
#define EVENT_TYPE_BLUETOOTH		9

// LCD: lcd on/off
// 	param: EVENT_FLAG_ON, EVENT_FLAG_TOGGLE, EVENT_FLAG_OFF
#define EVENT_TYPE_LCD			10

// SUSPEND
#define EVENT_TYPE_SUSPEND		11


///// --- event flags --- /////
#define EVENT_FLAG_INCREASE		0x1
#define EVENT_FLAG_DECREASE		0x2
#define EVENT_FLAG_ON			0x4
#define EVENT_FLAG_OFF			0x8
#define EVENT_FLAG_TOGGLE		0x10


///// --- private events --- /////
// PRIVATE_EVENT_TIMEOUT has a fixed mapping and cannot be used in event mapping below
#define PRIVATE_EVENT_TIMEOUT			1

#define PRIVATE_EVENT_BATT1_LOW			2
#define PRIVATE_EVENT_BATT1_FULL		3
#define PRIVATE_EVENT_BATT1_CONNECTED		4
#define PRIVATE_EVENT_BATT1_DISCONNECTED	5
#define PRIVATE_EVENT_BATT2_LOW			6
#define PRIVATE_EVENT_BATT2_FULL		7
#define PRIVATE_EVENT_BATT2_CONNECTED		8
#define PRIVATE_EVENT_BATT2_DISCONNECTED	9

#define PRIVATE_EVENT_AC_CONNECTED		10
#define PRIVATE_EVENT_AC_DISCONNECTED		11


#define EVENT_MAPPING \
	{SONYPI_EVENT_JOGDIAL_UP, EVENT_TYPE_BRIGHTNESS, (void*)EVENT_FLAG_INCREASE, EVENT_MODE_BRIGHTNESS}, \
	{SONYPI_EVENT_JOGDIAL_DOWN, EVENT_TYPE_BRIGHTNESS, (void*)EVENT_FLAG_DECREASE, EVENT_MODE_BRIGHTNESS}, \
	{SONYPI_EVENT_JOGDIAL_UP, EVENT_TYPE_VOLUME, (void*)EVENT_FLAG_INCREASE, EVENT_MODE_VOLUME}, \
	{SONYPI_EVENT_JOGDIAL_DOWN, EVENT_TYPE_VOLUME, (void*)EVENT_FLAG_DECREASE, EVENT_MODE_VOLUME}, \
 \
	{SONYPI_EVENT_FNKEY_ESC, EVENT_TYPE_EXIT, NULL, EVENT_MODE_ALL}, \
	{SONYPI_EVENT_FNKEY_F2, EVENT_TYPE_COMMAND, "echo -n  >/dev/console", EVENT_MODE_ALL}, \
	{SONYPI_EVENT_FNKEY_F3, EVENT_TYPE_MUTE, (void*)EVENT_FLAG_TOGGLE, EVENT_MODE_ALL}, \
	{SONYPI_EVENT_FNKEY_F4, EVENT_TYPE_CHANGE_MODE, (void*)EVENT_MODE_VOLUME, EVENT_MODE_ALL}, \
	{SONYPI_EVENT_FNKEY_F5, EVENT_TYPE_CHANGE_MODE, (void*)EVENT_MODE_BRIGHTNESS, EVENT_MODE_ALL}, \
	{SONYPI_EVENT_FNKEY_F7, EVENT_TYPE_LCD, (void*)EVENT_FLAG_TOGGLE, EVENT_MODE_ALL}, \
	{SONYPI_EVENT_FNKEY_F12, EVENT_TYPE_SUSPEND, NULL, EVENT_MODE_ALL}, \
	{SONYPI_EVENT_BLUETOOTH_PRESSED, EVENT_TYPE_BLUETOOTH, (void*)EVENT_FLAG_TOGGLE, EVENT_MODE_ALL}, \
 \
	{SONYPI_EVENT_LID_CLOSED, EVENT_TYPE_COMMAND, "echo -n  >/dev/console", EVENT_MODE_ALL}, \
 \
	{PRIVATE_EVENT_AC_DISCONNECTED, EVENT_TYPE_BRIGHTNESS_SET, (void*)0, EVENT_MODE_PRIVATE | EVENT_MODE_ALL}, \
	{PRIVATE_EVENT_AC_CONNECTED, EVENT_TYPE_BRIGHTNESS_SET, (void*)255, EVENT_MODE_PRIVATE | EVENT_MODE_ALL}, \
 \
	{0, EVENT_TYPE_NONE, NULL, 0}



////// --- function declarations --- /////

extern void handle_event(unsigned char event, bool private_event);


#endif
