/////////////////////////////////////////////////////////////////////
//  ivad - improved vaio daemon
//  	(C) BeF - fuhrmannek@web.de
//  	License: GPL
//
//  	have fun.
//
/////////////////////////////////////////////////////////////////////

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <linux/types.h>
#include <linux/sonypi.h>

#include "ivad.h"
#include "event_handler.h"
#include "sonypi_ctl.h"


#define OPTSTRING "dvs:m:h"

struct ivad_config_struct ivad_config;
int sonypi_fd;


void print_usage(const char *program_name)
{
	printf("Usage: %s [%s]\n\n", program_name, OPTSTRING);
	printf("\t%s\t%s\n", "-d", "aviod forking into daemon mode");
	printf("\t%s\t%s\n", "-v", "be very verbose");
	printf("\t%s\t%s\n", "-s <device>", "use <device> as the sonypi device");
	printf("\t%s\t%s\n", "-m <device>", "use <device> as the mixer device");
	printf("\t%s\t%s\n", "-h", "print this help");
}

void init_default_configuration()
{
	strcpy(ivad_config.sonypi_devicename, SONYPI_DEVICE);
	strcpy(ivad_config.mixer_devicename, MIXER_DEVICE);

	ivad_config.daemon_mode = true;
#ifdef DEBUG
	ivad_config.debug_mode = 1;
#else
	ivad_config.debug_mode = 0;
#endif

	ivad_config.brightness_step = BRIGHTNESS_STEP;
	ivad_config.mixer_step = MIXER_STEP;
	ivad_config.select_timeout = SELECT_TIMEOUT;
	ivad_config.low_battery = LOW_BATTERY;
}

void print_configuration()
{
	fprintf(stderr, "%s: %s\n", "sonypi device", ivad_config.sonypi_devicename);
	fprintf(stderr, "%s: %d\n", "daemon mode", ivad_config.daemon_mode);
	fprintf(stderr, "%s: %d\n", "brightness step", ivad_config.brightness_step);
	fprintf(stderr, "%s: %d\n", "select timeout", ivad_config.select_timeout);
}

void power_state_changes()
{
	static bool is_init = false;

	static __u8 batt_flags;
	static double batt1;
	static double batt2;

	__u8 new_batt_flags = get_battery_flags();
	double new_batt1 = get_battery_percent(1);
	double new_batt2 = get_battery_percent(2);

	if (is_init)
	{
		new_batt_flags = get_battery_flags();
		new_batt1 = get_battery_percent(1);
		new_batt2 = get_battery_percent(2);

		// compare old and new values
		//
		if (new_batt_flags & SONYPI_BFLAGS_B1)
		{
			if (!(batt_flags & SONYPI_BFLAGS_B1))
				handle_event(PRIVATE_EVENT_BATT1_CONNECTED, true);
			else
			{
				if (new_batt1 <= ivad_config.low_battery && batt1 > ivad_config.low_battery)
					handle_event(PRIVATE_EVENT_BATT1_LOW, true);
				else if (new_batt1 == 1.0 && batt1 < 1.0)
					handle_event(PRIVATE_EVENT_BATT1_FULL, true);
			}
		}
		else if (batt_flags & SONYPI_BFLAGS_B1)
			handle_event(PRIVATE_EVENT_BATT1_DISCONNECTED, true);
		
		if (new_batt_flags & SONYPI_BFLAGS_B2)
		{
			if (!(batt_flags & SONYPI_BFLAGS_B2))
				handle_event(PRIVATE_EVENT_BATT2_CONNECTED, true);
			else
			{
				if (new_batt2 <= ivad_config.low_battery && batt2 > ivad_config.low_battery)
					handle_event(PRIVATE_EVENT_BATT2_LOW, true);
				else if (new_batt2 == 1.0 && batt2 < 1.0)
					handle_event(PRIVATE_EVENT_BATT2_FULL, true);
			}
		}
		else if (batt_flags & SONYPI_BFLAGS_B2)
			handle_event(PRIVATE_EVENT_BATT2_DISCONNECTED, true);

		if (new_batt_flags & SONYPI_BFLAGS_AC)
		{
			if (!(batt_flags & SONYPI_BFLAGS_AC))
				handle_event(PRIVATE_EVENT_AC_CONNECTED, true);
		}
		else if (batt_flags & SONYPI_BFLAGS_AC)
			handle_event(PRIVATE_EVENT_AC_DISCONNECTED, true);

		batt_flags = new_batt_flags;
		batt1 = new_batt1;
		batt2 = new_batt2;
	}
	else
	{
		// initialization
		batt_flags = get_battery_flags();
		batt1 = get_battery_percent(1);
		batt2 = get_battery_percent(2);
		is_init = true;
	}

	
}

// prerequisites:
// 	sonypi_fd must be the valid file descriptor of
// 	the r/w opened sonypi device
void event_loop()
{
	unsigned char event;

	fd_set rdfs;
	struct timeval tv;

	power_state_changes();

	FD_ZERO(&rdfs);

	while ( 1 )
	{
		FD_SET(sonypi_fd, &rdfs);	

		tv.tv_sec = ivad_config.select_timeout;
		tv.tv_usec = 0;

		if (select(sonypi_fd + 1, &rdfs, NULL, NULL, &tv))
		{
		       if (FD_ISSET(sonypi_fd, &rdfs))
		       {
				if (read(sonypi_fd, &event, sizeof(event)) != sizeof(event))
					fprintf(stderr, "Error: unable to read the important data\n");
				else
					handle_event(event, false);
		       }
		}
		else
		{
			handle_event(PRIVATE_EVENT_TIMEOUT, true);
			power_state_changes();
		}
	}
}



int main(int argc, char **argv)
{
	int opt;
	bool fork_mode = true;
	
	printf("improved vaio daemon - version %s\n\t%s\n", IVAD_VERSION, AUTHOR);

	init_default_configuration();
	
	while ((opt = getopt(argc, argv, OPTSTRING)) >= 0)
	{
		switch (opt)
		{
			case 'd':
				fork_mode = false;
				break;
			case 'v':
				ivad_config.debug_mode = true;
				break;
			case 's':
				strcpy(ivad_config.sonypi_devicename, optarg);
				break;
			case 'm':
				strcpy(ivad_config.mixer_devicename, optarg);
				break;
			case 'h':
				print_usage(argv[0]);
				_exit(1);
			case '?':
			case ':':
				_exit(1);
		}
	}
	
	if (ivad_config.debug_mode)
		print_configuration();

	nice(10);

	if (fork_mode && ivad_config.daemon_mode && fork() > 0)
		_exit(0);

	if ((sonypi_fd = open(ivad_config.sonypi_devicename, O_RDWR)) == -1)
	{
		fprintf(stderr, "Error opening device %s: %s\n", ivad_config.sonypi_devicename, strerror(errno));
		return -1;
	}

	if (ivad_config.daemon_mode)
		event_loop();
	else
	{
	}

	// exit automatically closes any open file descriptors
	// 	possibly open fds are: sonypi_fd and mixer_fd
	_exit(0); 
}
